@@ -82,7 +82,7 @@ sub read
 {
     my ($self) = @_;
     my ($dat, $i, $j, $k, $id, @ids, $s);
-    my ($start, $end, $range, $delta, $form, $len, $num, $ver);
+    my ($start, $end, $range, $delta, $form, $len, $num, $ver, $sg);
     my ($fh) = $self->{' INFILE'};
 
     $self->SUPER::read or return $self;
@@ -164,9 +164,9 @@ sub read
             $fh->read($dat, 12 * $num);
             for ($j = 0; $j < $num; $j++)
             {
-                ($start, $end, $s) = unpack("N3", substr($dat, $j * 12, 12));
+                ($start, $end, $sg) = unpack("N3", substr($dat, $j * 12, 12));
                 for ($k = $start; $k <= $end; $k++)
-                { $s->{'val'}{$k} = $s++; }
+                { $s->{'val'}{$k} = $sg++; }
             }
         } elsif ($form == 10)
         {
@@ -206,7 +206,7 @@ to it if found. Returns the table it finds.
 sub find_ms
 {
     my ($self) = @_;
-    my ($i, $s, $alt);
+    my ($i, $s, $alt, $found);
 
     return $self->{' mstable'} if defined $self->{' mstable'};
     $self->read;
@@ -216,10 +216,12 @@ sub find_ms
         if ($s->{'Platform'} == 3)
         {
             $self->{' mstable'} = $s;
-            last if ($s->{'Encoding'} == 1);
+            last if ($s->{'Encoding'} == 10);
+            $found = 1 if ($s->{'Encoding'} == 1);
         } elsif ($s->{'Platform'} == 0 || ($s->{'Platform'} == 2 && $s->{'Encoding'} == 1))
-        { $self->{' mstable'} = $s; }
+        { $alt = $s; }
     }
+    $self->{' mstable'} = $alt if ($alt && !$found);
     $self->{' mstable'};
 }
 
@@ -261,6 +263,12 @@ sub out
 
     return $self->SUPER::out($fh) unless $self->{' read'};
 
+
+    $self->{'Tables'} = [sort {$a->{'Platform'} <=> $b->{'Platform'}
+                                || $a->{'Encoding'} <=> $b->{'Encoding'}
+                                || $a->{'Ver'} <=> $b->{'Ver'}} @{$self->{'Tables'}}];
+    $self->{'Num'} = scalar @{$self->{'Tables'}};
+
     $base_loc = $fh->tell();
     $fh->print(pack("n2", 0, $self->{'Num'}));
 
@@ -358,10 +366,13 @@ sub out
                 }
                 if ($j != $current + 1 || $s->{'val'}{$j} != $curr_glyf + 1)
                 {
-                    push (@jobs, [$start, $current, $curr_glyf]) if (defined $start);
+                    push (@jobs, [$start, $current, $curr_glyf - ($current - $start)]) if (defined $start);
                     $start = $j; $current = $j; $curr_glyf = $s->{'val'}{$j};
                 }
+                $current = $j;
+                $curr_glyf = $s->{'val'}{$j};
             }
+            push (@jobs, [$start, $current, $curr_glyf - ($current - $start)]) if (defined $start);
             $fh->print($map) if ($s->{'Format'} == 8);
             $fh->print(pack('N', $#jobs + 1));
             foreach $j (@jobs)
@@ -412,21 +423,38 @@ sub XML_element
     $self;
 }
 
-=head2 @map = $t->reverse([$num])
+=head2 @map = $t->reverse(%opt)
 
-Returns a reverse map of the table of given number or the Unicode
-cmap. I.e. given a glyph gives the Unicode value for it.
+Returns a reverse map of the Unicode cmap. I.e. given a glyph gives the Unicode value for it. Options are:
+
+=over 4
+
+=item tnum
+
+Table number to use rather than the default Unicode table
+
+=item array
+
+Returns each element of reverse as an array since a glyph may be mapped by more
+than one Unicode value. The arrays are unsorted.
+
+=back
 
 =cut
 
 sub reverse
 {
-    my ($self, $tnum) = @_;
-    my ($table) = defined $tnum ? $self->{'Tables'}[$tnum] : $self->find_ms;
+    my ($self, %opt) = @_;
+    my ($table) = defined $opt{'tnum'} ? $self->{'Tables'}[$opt{'tnum'}] : $self->find_ms;
     my (@res, $code, $gid);
 
     while (($code, $gid) = each(%{$table->{'val'}}))
-    { $res[$gid] = $code unless (defined $res[$gid] && $res[$gid] > 0 && $res[$gid] < $code); }
+    {
+        if ($opt{'array'})
+        { push (@{$res[$gid]}, $code); }
+        else
+        { $res[$gid] = $code unless (defined $res[$gid] && $res[$gid] > 0 && $res[$gid] < $code); }
+    }
     @res;
 }
 
